#define PYGII_MODULE
#include "pygii.h"

static PyObject *  timeout;
static pygii_C_api capi;

PyObject *
pygii_newEventFrom(gii_event ev) {
	Event *r;
	/* Create an event object */
	r = PyObject_New(Event, &EventType);
	if (r)
		r->ev = ev;
	return (PyObject *)r;
}

int
pygii_getInt(PyObject *o, int *i) {
	int a;
	a=PyInt_AsLong(o);
	if(PyErr_Occurred())
		return -1;
	*i=a;
	return 0;
}

int
pygii_getTime(PyObject *o, struct timeval * t) {
	double f;
	f=PyFloat_AsDouble(o);
	if(PyErr_Occurred())
		return -1;
	FLOAT_TO_TS(t,f);
	return 0;
}

PyObject *
pygii_getDeviceInfo(struct gii_input * inp) {
	struct gii_input * curr;
	gii_deviceinfo   *di;
	PyObject * ret;
	
	curr=inp;
	ret=PyList_New(0);
	do {
		for(di=curr->devinfo;di;di=di->next)			
			PyList_Append(ret,Py_BuildValue(
					      "(iissiii)",
					      curr->origin,di->origin,
					      di->dev->shortname,
					      di->dev->longname,
					      di->dev->can_generate,
					      di->dev->num_buttons,
					      di->dev->num_axes));		
		curr = curr->next;
	} while(curr != inp);	/* looped through once. */
	return ret;
}

void
pygii_raiseTimeout(void) {
	PyErr_SetObject(timeout, Py_None);
}

static struct PyMethodDef giimodule_methods[] = {
	{ NULL, NULL, 0, (char*)NULL }
};

static void importModules(void) {
        PyObject *g, *d;
        
        g = PyImport_ImportModule("GGI.exception");
        if (g!= NULL) {
                d = PyModule_GetDict(g);
		timeout = PyDict_GetItemString(d, "Timeout");
		/*		Py_DECREF(d);
				Py_DECREF(g); */
        } else {
                Py_FatalError("Can't import GGI.exception");
                return;
        }
}

#ifndef PyMODINIT_FUNC
#define PyMODINIT_FUNC void
#endif
PyMODINIT_FUNC
init_giimodule(void)  {	
	PyObject *m;

	/* Initialize the types */ 
	
	if(PyType_Ready(&EventType) < 0) return;
	if(PyType_Ready(&InputType) < 0) return;
	
	importModules();

	m = Py_InitModule3("_giimodule", giimodule_methods, (char*)NULL);
	
	Py_INCREF(&EventType);
	PyModule_AddObject(m, "Event", (PyObject*)&EventType);
	Py_INCREF(&InputType);
	PyModule_AddObject(m, "Input", (PyObject*)&InputType);
	
	/* register C API */
	
	capi.InputType = &InputType;
	capi.EventType = &EventType;
	
	capi.newEventFrom  = &pygii_newEventFrom;
	capi.getDeviceInfo = &pygii_getDeviceInfo;
	capi.raiseTimeout  = &pygii_raiseTimeout;
	
	PyModule_AddObject(m, "_giimodule_C_api",
			   PyCObject_FromVoidPtr(&capi, NULL));
	
}
